#   AllPlotAPI.py - API for AllPlot
'''
    Here's a basic example using the AllPlot API:

import sys
sys.path.append("/path/to/AllPlot.zip")
import AllPlotAPI as ap
inp = ap.TextInputFile("filename")  # contains column "x"
win = ap.Window(inp)
ap.Histogram(win,"x")
ap.displayAll()
# displayAll runs until all AllPlot windows are closed.

'''

import textinput, numpyinput, window, plot, input, tools
try: 
    import root
    RootInput = True
except:
    RootInput = False

###
### First, the user must import this API; by convention it is named "ap"
### (by using append this also works in the LoadConfig menu item)
### (doing this multiple times is OK, so SaveConfig can simply append to file)
###
###     import sys
###     sys.path.append("/path/to/AllPlot.zip")
###     import AllPlotAPI as ap

###
### Second, an input is needed
###

#   inp = ap.TextInputFile(filename, minEvent=0, maxEvent=1000000)
TextInputFile = textinput.TextInputFile

#   input = ap.NumpyInputFile(filename)   # no minEvent/maxEvent
NumpyInputFile = numpyinput.NumpyInputFile

#   input = ap.RootInputFile(filename,ntuplePath)   # no minEvent/maxEvent
if RootInput:
    RootInputFile = root.RootInputFile

#   inp = ap.Generate()
#   inp.setSamples(nSamples, sampleRate)
#   inp.addDCOffset(value)
#   inp.addSinewave(amplitude, frequency, phase)
#   inp.addGaussianNoise(mean, sigma)
#   inp.addWhiteNoise(amplitude)
#   inp.goGenerate()
Generate = input.Generate

###
### Third, a Window is needed for the input
###

#   win = ap.Window(inp)
Window = window.Window

###
### Fourth, every plot needs a Style and an AxesParam
### (if either is None, sensible defaults are used)
###
#   style = ap.Style(color='blue', linestyle='solid', pointMarker=' ')
Style = plot.Style

#   axesparam = ap.AxesParam(title="", xlabel="", ylabel="", logx=False,
#                   logy=False, gridlines=True, 
#                   xmin=None, xmax=None, ymin=None, ymax=None)
#   if the last four limits are all None, autoscale is on.
AxesParam = plot.AxesParam

###
### Fifth, create plots in the Window
###

#   ap.SelectionPlot(win, columnName, style=None)   # only default AxesParam
SelectionPlot = plot.SelectionPlot

#   ap.SimpleDataPlot(win, columnName, style=None)  # only default AxesParam
SimpleDataPlot = plot.SimpleDataPlot

#   ap.Histogram(win, x, w="", cut="", nbins=100, errorbars=False,
#                                                style=None, axesparam=None)
Histogram = plot.Histogram

#   ap.XYPlot(win, x, y,  cut='', xerrbar='', yerrbar='', style=None,
#                                                            axesparam=Nonea)
XYPlot = plot.XYPlot

#   ap.FFTPlot(win, x, sampleRate=1000, windowFunction=None, style=None,
#                                                            axesparam=None)
FFTPlot = plot.FFTPlot

#   ap.CorrelationPlot(win, x1, x2, subtractMeans=False, extendX1=False,
#                                    style=None, axesparam=None)
CorrelationPlot = plot.CorrelationPlot

#   ap.AllanDevPlot(win, x, sampleRate, style=None, axesparam=None)
AllanDevPlot = plot.AllanDevPlot

#   ap.QuadPlot(win, columnName)
QuadPlot = plot.QuadPlot

#   ap.ComboPlot(win, plotList)
ComboPlot = plot.ComboPlot

#   ap.ComparePlot(win, plotlist, commonX=False, xmin=None, xmax=None,
#                                                style=None, axesparam=None)
ComparePlot = plot.ComparePlot

# Optional:  fit an expression in x to a plot interpreted as y(x)
# fit = ap.Fit(plot, expression, method="Nelder-Mead", printDetails=False)
# fit.param("name1", value1, fixed=False, step=None, min=None, max=None)
# fit.param("name2", value2, fixed=False, step=None, min=None, max=None)
# fit.param("name3", value3, fixed=False, step=None, min=None, max=None)
# ...
# fit.doFit()    # returns True if converged, False otherwise
# fit.getValues()   # returns a dict with Chisq,NDF,Iterations and param values
Fit = tools.Fit

###
### Sixth, the user must call ap.displayAll() to display the windows and
### plots. Does not return until all windows are closed and the program is 
### exiting.
###
displayAll = window.displayAll


###
### global stuff -- here we call window.setup() to prepare for API calls.
### This works for both stand-alone and File/LoadConfig.
###
window.setup()

