#!/usr/bin/env python3
#   __main__.py - startup module for Python 3
#
#   If run via python 2.X, this program searches for python3 in PATH
#   and executes it. If not found, prints an error and exits.
#   Then it imports all modules, printing an error message for the first
#   missing module, and a nots about what modules are needed.

from __future__ import print_function
import sys, os, platform, time
# more imports below, for managing missing module error message

if sys.version_info[0] != 3:
    prog = "python3"
    if platform.system() == "Windows":
        prog = "python3.exe"
    file = __file__
    i = file.find("zip/__main__.py")
    if i > 0: file = file[:i+3]
    for dir in os.environ["PATH"].split(os.pathsep):
        fn = os.path.join(dir,prog)
        if os.path.isfile(fn) and os.access(fn, os.X_OK):
            s = fn + " " + file
            print("Running "+s)
            i = os.system(s)
            sys.exit(i)
    print("Cannot find Python3 - exiting")
    print("(sleep for 10 seconds)")
    time.sleep(10)
    sys.exit(99)


###
### Import all modules to see if they are present; print error if not
###
try:
    # be sure to import all modules used anywhere, so the error msg is complete
    import re, math, sys, time, os, io, queue, pathlib, platform
    import threading, copy, atexit, types
    import os.path as path
    import tkinter as tk
    from tkinter import ttk
    from tkinter import messagebox
    from tkinter import simpledialog
    from tkinter import scrolledtext
    import tkinter.filedialog as filedialog
    import matplotlib
    matplotlib.use("TkAgg")
    import matplotlib.pyplot as plt
    import numpy as np
except ModuleNotFoundError as e:
    print("AllPlot FATAL ERROR: ",e)
    print("The following standard Python modules are used:")
    print("    sys time re os math os.path io queue pathlib platform")
    print("    threading copy atexit types")
    print("You must install the following Python modules for your platform:")
    print("    tkinter, matplotlib, numpy")
    print("In addition, the following optional modules will improve AllPlot:")
    print("    pyroot      read Root files")
    print("    pandas      speed up reading text files")
    print("    scipy       enable fitting to plots")
    print("    allantools  enable plotting Allan Deviation")
    print("(sleep for 10 seconds)")
    time.sleep(10)
    sys.exit(2)

if __name__ == "__main__":
    import window
    window.main()
