#   help.py - AllPlot help, and AllPlot API help.

def helpAllPlot():
    return '''
                                AllPlot 3.0
                    Copyright (C) 2020-23 by Tom Roberts.
                            All rights reserved.

                    This program is Open Source and may be
                    redistributed in accordance with the Gnu
                    Public License v3.

AllPlot is a general-purpose plotting program designed to read common data
file formats and to easily display plots that are commonly used in physics
analyses.

Data file formats (described below):
    text   space/tab separated (.txt) or comma separated (.csv)
    numpy  (.npz)
    Root   (.root)    (you must install pyroot)

A data file consists of multiple rows (events), each with the same number of
columns (fields). Columns have names, and many plots can display arithmetic
expressions involving those names and common math functions. The column 'N'
is generated for each data file, containing the row number for each row
(starting with 0). If column names are not found in the file, 'c1', 'c2',
... are used.

Each data file is displayed in its own window; each plot is displayed in a
tab of its data-file window. The tab can be clicked to select it and display
its plot; Ctrl-Tab and Ctrl-Shift-Tab move between tabs (and their plots).

Plots:
    Selection Plot      A plot of one column vs Time (if present) or N.
                        The middle mouse button can be clicked and dragged
                        to define a range in Time or N; all other plots in
                        this window will use only the data in that range.
    Simple Data Plot    Plot one column vs Time (if present) or N.
    Histogram           Histogram of an expression, with optional
                        statistical errorbars, weight, and cut (logical
                        expression).
    XY plot             Plot one expression vs another expression, with
                        optional cut and errorbars. For a scatterplot set
                        its style to use no line and an appropriate marker.
    FFT                 Fast Fourier Transform of an expression, with
                        optional windowing. A Time column is required.
    Correlation         Plot the correlation of one expression with another.
    Allan Deviation     Plot the Allan deviation of an expression.
                        A Time column is required.

The above plots show a single plot; the following combine multiple plots:
    Combo Plot          Plot multiple existing plots on a single axes.
                        Each plot uses its current style.
    Quad Plot           Plot four common plots of one column: Data, FFT,
                        Histogram, and Allan Deviation. A Time column is
                        required.
    Compare Plot        Plot up to 8 plots vertically in a single frame,
                        with X axes aligned. Each plot uses a separate
                        axes and its current style.

Each plot has an editable Style (color, solid/dotted/dashed, point markers),
with title and labels for X and Y axes, optional log scaling for X and Y
axes, and optional gridlines. Each plot can be auto-scaled (default), or
the user can set specific limits in X and Y.

In most plots, the user can left-click and drag to define a rectangle, and
the plot will be zoomed-in to it. Successive zooms are remembered, and 
Right-Click will pop back one level.

Arithmetic functions usable in expressions: abs, arccos, arccosh, arcsin,
arcsinh, arctan, arctan2, arctanh, min, max, ceil, floor, cos, cosh,
deg2rad, degrees, exp, exp2, fabs, log, log10, log2, power, rad2deg, sin,
sinh, sinc, sqrt, tan, tanh, hypot.

This program is written in Python 3, and extensively uses Numpy arrays and
array operations for efficiency. Note that all data files must fit in 
memory simultaneously, as well as intermediate array values.

Support
-------
Questions about AllPlot can be discussed in the G4beamline forum. You may 
request to join the forum: mailto:g4beamline+subscribe@muonsinc.com.

AllPlot can be downloaded from http://allplot.muonsinc.com

NOTE: installing Pandas will speed up reading text files by more than a
factor of 10, and is VERY worthwhile if you use large files. If Pandas is
not installed, a slower numpy routine is used.
	pip3 install pandas

Optional features:
    To enable Allan Deviation plots:
        pip3 install allantools
    To enable fitting to plots:
    	pip3 install scipy
    To read Root Files:
        install Root in the usual way for your platform --
        that should include the ROOT Python module, pyroot.
	If not, try: pip3 install pyroot
	On macOS, if you installed Python3 via HomeBrew, install
	Root via HomeBrew as well, or it won't install pyroot into
	Python3.
    To use the IIT ADC Box, the IIT extensions must be present. Then:
	pip3 install multiprocessing. AllPlot detects if the
	ADC box is physically connected, and omits it if not.

File Formats
------------
Text File
The "Text File Input" automatically detects whether the file is space
delimited (typically .txt) or comma delimited (typically .csv). This is
determined by the file contents, not the extension (which can be anything).

Text File, Space Delimited:
Columns are delimited by one or more spaces and tabs. Comment lines begin
with '#'. If the file begins with a series of comments, the file is read
until the first non-comment line, which determines whether it is space-
or comma-delimited, and also the number of columns. The column names come
from the first initial comment line containing the same number of names,
each of which is a valid Python identifier. If column names are not found,
"c1", "c2", "c3", ... are used.

Text File, Comma Delimited:
When the first non-comment line in the file is comma delimited, it can
contain column names enclosed in single- or double-quotes; each must be a
valid Python identifier. If column names are not found, "c1", "c2", "c3", ...
are used.

Numpy file:
Only Numpy zip files can be read. Their format is a dictionary with key
being the column name and value being a 1-d Numpy array with dtype=float32 or
dtype=float64. All columns must have the same length > 1. There can also be
parameters in the dictionary (value is a single float32 or float64), and a
_comment (value is a string of arbitrary length).

Root File:
AllPlot will read only TNtuple-s from a Root TFile. They inherently have 
named columns (fields). This can be particularly useful for reading Root
files written by G4beamline. See the Root documentation for details.

'''

def helpAllPlotAPI():
    return '''
                                AllPlot 3.0
                    Copyright (C) 2020-22 by Tom Roberts.
                            All rights reserved.

                    This program is Open Source and may be
                    redistributed in accordance with the Gnu
                    Public License v3.

AllPlot is a general-purpose plotting program designed to read common data
file formats and to easily display plots that are commonly used in physics
analyses.

The AllPlot API permits the user to write a Python program that creates
AllPlot windows, plots, and fits. The simplest way to create such a 
program is via File/SaveConfig, which writes the current AllPlot configuration
as a Python program using the API.

The user program can create multiple Windows, each with its own Input. Each
Window can have multiple Plots, displayed as Tabs in the Window. Most Plots
use arbitrary arithmetical expressions involving the column names of its
Window's Input.

AllPlot considers each dataset to consist of multiple rows (events), each
containing some number of columns (fields). Each column has a name, and
every row must contain all the columns. Input files normally contain the
names of their columns; if column names are not present, "c1", "c2", "c3",
... are used.

***
*** Note: to distinguish text from code, here the code is indented.
*** The actual Python file, of course, must have no indent.
***

First, the AllPlot ZIP must be added to the Python path:
    import sys
    sys.path.append("/path/to/AllPlot.zip")
Now any module from AllPlot can simply be imported by name (AllPlot analysis
iit input numpyinput plot root textinput tools window).
By using an absolute path for AllPlot.zip, the user file can be placed
anywhere, presumably close to the data being plotted. On Windows use
"C:\\path\\to\\AllPlot.zip". Of course if the AllPlot zip is in the same
folder as the Python program, no path is needed

In particular, the API should be imported first:
    import AllPlot as ap
The convention is to import AllPlot as ap; it will be used throughout.

Second, an input is needed. It will read a file or generate data:
    # To read a space-separated or comma-separated text file:
    inp = ap.TextInputFile(filename, minEvent=0, maxEvent=1000000)
or
    # to read a .npz file:
    inp = ap.NumpyInputFile(filename)
or
    # to read a TNtuple from a Root TFile (if Root is installed):
    inp = ap.RootInputFile(filename,ntuplePath)
or
    # To generate time-series data:
    inp = ap.Generate()
    inp.setSamples(nSamples, sampleRate)
    # Any number of the various addX() functions can be used:
    inp.addDCOffset(value)
    inp.addSinewave(amplitude, frequency, phase)
    inp.addGaussianNoise(mean, sigma)
    inp.addWhiteNoise(amplitude)
    # finally, doGenerate() must be called to create "N", "Time", and "Data"
    inp.goGenerate()
The convention is to use inp as the local variable for the Input; it will
be referenced when creating a Window for the Input.

***
*** Note: the column "N" will always be created, containing the row (event)
*** number, starting at 0.
***

There are several useful functions available:
    inp.generateTimeColumn(sampleRate)
    inp.setSelectionPlotColumnName(columnName)

Third, a Window is needed for the input:
    win = ap.Window(inp)
The convention is to use win as the local variable for the Window; it will
be referenced by each plot.

The Window corresponds to the input, and can have multiple plots. It will
have the usual AllPlot menu, to which you can add pull-down menus in the
usual way. There can be multiple Windows, with different or the same Input.

Fourth, each plot needs a Style and an AxesParam; Style controls how the
plot looks, while AxesParam controls the axes. If either is None, sensible
defaults are used, so if the defaults are acceptable, you can omit these
and the corresponding arguments to the plots.
    style = ap.Style(color='blue', linestyle='solid', pointMarker=' ')
    axesparam = ap.AxesParam(title="", xlabel="", ylabel="",
                    logx=False, logy=False, gridlines=True,
                    xmin=None, xmax=None, ymin=None, ymax=None)
If the last four limits are None, autoscale is on and the axes limits will
be determined by the data plotted.

***
*** Note: matplotlib has a bug that makes fixed limits not work with
*** any log axis, so use autoscale for such plots.
***

Fifth, create the desired plots in the current Window, using the data from
its Input. Use any of the following:
    p1 = ap.SimpleDataPlot(win, columnName, style=None)
    p1 = ap.Histogram(win, x, w="", cut="", nbins=100, errorbars=False,
                                                style=None, axesparam=None)
    p1 = ap.XYPlot(win, x, y,  cut='', xerrbar='', yerrbar='', style=None,
                                                            axesparam=None)
    p1 = ap.FFTPlot(win, x, sampleRate=1000, windowFunction=None, style=None,
                                                            axesparam=None)
    p1 = ap.CorrelationPlot(win, x1, x2, subtractMeans=False, extendX1=False,
                                                style=None, axesparam=None)
    p1 = ap.AllanDevPlot(win, x, sampleRate, style=None, axesparam=None)
    p1 = ap.QuadPlot(win, columnName)
    p1 = ap.ComboPlot(win, plotList)
    p1 = ap.ComparePlot(win, plotlist, commonX=False, xmin=None, xmax=None)
Here x,y,w,xerrbar,yerrbar are all expressions involving the columnNames of
the input, and standard functions. cut is a logical expression that
determines which events (rows) of the input will be included in the plot.
Help for each plot type can be obtained from the GUI.

***
*** NOTE: the cut expression must conform to the syntax of Python NumPy
*** array expressions. So it should combine parenthesized comparisons with
*** & (and) and | (or). For example (the data has columns x and y):
***     (x >= 0.0) & (x < 10.0) & (y >= 0.0) & (y < 10.0)
***

Here p1 is used as the local variable, but the number should be the plot
number. The local variables are needed only for plots with a "plotList"
argument (ComboPlot and ComparePlot); if you don't use them you can omit
the "p1 =". For example, to combine the first three plots your code would
set p1, p2, and p3 to plots and use them in the ComboPlot:
    p4 = ap.ComboPlot(win,[p1,p2,p3])
When using ComboPlot, it is desirable to use Styles with different colors
for the plots. In the GUI you can change the Style of a plot, then select
the ComboPlot and do File/Replot to use its new Style.

Optional: fit an expression in x to a plot interpreted as y(x):
    fit = ap.Fit(plot, expression, method="Nelder-Mead", printDetails=False)
    fit.param("name1", value1, fixed=False, step=None, min=None, max=None)
    fit.param("name2", value2, fixed=False, step=None, min=None, max=None)
    fit.param("name3", value3, fixed=False, step=None, min=None, max=None)
    ...
    fit.doFit()       # returns True if converged, False otherwise
    fit.getValues()   # returns a dict with entries for Chisq, NDF,
                      # Iterations and each param value.
The expression must be a parameterized expression in x, and the fit will
minimize the Chisq between it and the data in the plot (interpreted as y(x)).
The result of the fit will be drawn on the plot using a solid red line.
Other minimization methods are available, see the GUI for a list.
printDetails=True will print a line for each iteration of the fit, plus the
final Hessian matrix, etc.  For example, to fit a quadratic to plot p1:
    fit = ap.Fit(p1,"a*x**2+b*x+c")
    fit.param("a",1.0)
    fit.param("b",0.0)
    fit.param("c",0.0)
    fit.doFit()
    print(fit.getValues())

Finally, once all Windows and Plots have been created, this function must be
called to actually display the windows and plots by executing the tkinter
mainloop():
    ap.displayAll()
It does not return until all Windows are closed and the program is exiting.

'''
