#   numpyinput.txt

import sys, time, re, os, math
import tkinter as tk
from tkinter import ttk
from tkinter import filedialog
from tkinter import messagebox
from tkinter import simpledialog
import numpy as np
from copy import *

from input import InputParameters, Input

###
### class NumpyInputParameters
###
class NumpyInputParameters(InputParameters):
    def isValid(self):
        return True
    def addTab(self,tabs):
        self.parent = tabs.master
        self.input = None
        f = tk.Frame(tabs)
        tabs.add(f,text="Numpy File",padding="0.1i")
        tk.Label(f,text="Numpy File Input").pack()
        tk.Label(f,text="").pack()
        tk.Button(f,text="Select Numpy File",command=self.selectNumpyFile). \
                                                pack()
        self.textFilename = tk.Label(f,text="")
        self.textFilename.pack(fill=tk.X)
        ff = tk.Frame(f)
        self.generateTimeVar = tk.IntVar()
        self.generateTimeVar.set(0)
        self.generateTime = tk.Checkbutton(ff,
                                text="Generate Time, sample rate:",
                                variable=self.generateTimeVar)
        self.generateTime.pack(side=tk.LEFT)
        self.sampleRate = tk.Entry(ff,width=6)
        self.sampleRate.insert(tk.END,"1000")
        self.sampleRate.pack(side=tk.LEFT)
        tk.Label(ff,text=" Hz").pack(side=tk.LEFT)
        ff.pack()
        tk.Label(f,text=" ").pack()
        tk.Label(f,text="A Selection Plot will be displayed showing" + \
                                " one column:").pack()
        ff = tk.Frame(f)
        self.col = ttk.Combobox(ff,width=15,values=[],state="readonly")
        self.col.pack(side=tk.LEFT)
        ff.pack()
        tk.Label(f,text="Then other plots can be displayed.").pack()
        tk.Label(f,text=" ").pack()
    def getInput(self):
        if self.input is None: return None
        if self.generateTimeVar.get() != 0:
            self.input.setSampleRate(int(self.sampleRate.get()))
        self.input.setSelectionPlotColumnName(self.col.get())
        return self.input
    def selectNumpyFile(self):
        self.generateTime.configure(state=tk.NORMAL)
        self.sampleRate.configure(state=tk.NORMAL)
        self.col.configure(values=[])
        self.col.set("")
        filename = filedialog.askopenfilename(parent=self.parent)
        if filename is None: filename = ""
        if not filename == "":
            self.input = NumpyInputFile(filename)
            if self.input.isValid():
                colnames = copy(self.input.columnNames())
                if colnames.count("Time") > 0:
                    colnames.remove("Time")
                    self.generateTimeVar.set(0)
                    self.generateTime.configure(state=tk.DISABLED)
                    self.sampleRate.configure(state=tk.DISABLED)
                colnames.insert(0,"(none)")
                self.col.configure(values=colnames)
                self.col.current(1)
            else:
                messagebox.showerror(title="AllPlot Error",
                        message="Invalid NumPy input file '"+filename+"'")
                filename = ""
        self.textFilename.configure(text=filename)

###
### class NumpyInputFile - read a Numpy file
###
class NumpyInputFile(Input):
    def __init__(self, filename):
        Input.__init__(self)
        self.filename = filename
        self.shortFilename = os.path.basename(filename)
        self.data = None
        self.read()  # read to get column names
    def read(self):
        if self.data is not None: return
        save = self.saveAllParams()
        try:
            # NpzFile object cannot be modified, so
            # convert to an ordinary dictionary object.
            self.data = dict(np.load(self.filename))
            self.inp = "Numpy input file"
        except:
            self.data = None
            self.inp = None
            return
        if "N" not in self.data:
            # get largest array length (because _markers could be present)
            k = 2  # ignore parameters and _comment (which have len == 1)
            for n in self.data.keys():
                if len(self.data[n]) > k: k = len(self.data[n])
            self.data["N"] = np.arange(k)
        self.restoreAllParams(save)
        print("Numpy Input Filename: "+self.shortFilename)
        print("Numpy Input Columns: "+",".join(self.columnNames()))
        print("Numpy Input Parameters: "+",".join(self.parameterNames()))
    def toAPIString(self):
        ret = "\n"
        ret += "inp = ap.NumpytInputFile('%s')\n" % (self.filename)
        return ret

